/**
 * 
 */
package chat.client;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.PrintStream;
import java.net.Socket;
import java.net.UnknownHostException;
import java.util.concurrent.Executors;

/**
 * Classe fournissant les fonctionalités de base d'un client
 *
 */
public abstract class AbstractClient {
    // pseudonyme de l'utilisateur
    private String alias;

    // socket de communication avec le serveur
    private Socket socket;
    // flot de sortie associé au socket
    private PrintStream outputStream;
    // flot d'entrée associé au socket
    private BufferedReader inputStream;
    private volatile boolean running = false;

    /**
     * Constructeur
     * 
     * @param server
     *            nom d'hôte ou adresse IP du serveur
     * @param port
     *            numéro de port sur lequel le serveur écoute
     * @param alias
     *            pseudonyme de l'utilisateur
     * @throws UnknownHostException
     * @throws IOException
     */
    public AbstractClient(String server, int port, String alias) throws UnknownHostException, IOException {
        this.socket = new Socket(server, port);
        this.outputStream = new PrintStream(this.socket.getOutputStream());
        this.inputStream = new BufferedReader(new InputStreamReader(this.socket.getInputStream()));
        this.alias = alias;
    }

    /**
     * Affichage d'un message
     * 
     * @param message
     *            message à afficher
     */
    public abstract void displayMessage(String message);

    /**
     * Affichage d'un message d'erreur
     * 
     * @param message
     *            message d'erreur à afficher
     */
    public abstract void displayError(String message);

    /**
     * Envoie un message au serveur
     * 
     * @param message
     *            message à envoyer
     */
    public void sendToServer(String message) {
        outputStream.println(this.alias+" > "+message);
    }

    /**
     * Démarre le client
     */
    public void start() {
    	Executors.newSingleThreadExecutor().submit(new Runnable() {

    		@Override
    		public void run() {
    			try {
    				AbstractClient.this.running = true;
    				while (AbstractClient.this.running) {
    					String message;
    					try {
    						message = AbstractClient.this.inputStream.readLine();
    						if (message != null)
    							processMessage(message);
    					}
    					catch (Exception e) {
    						e.printStackTrace();
    					}
    				}
    			}
    			finally {
    				try {
    					AbstractClient.this.socket.close();
    				}
    				catch (IOException e) {
    					e.printStackTrace();
    				}

    			}
    		}
    	});
    }
    
    /**
     * Méthode qui traite un message reçu du serveur
     * @param message message à traiter
     */
    private void processMessage(String message) {
        // par défaut, affichage du message
        displayMessage(message);
    }
}
